#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Created by Roberto Preste
import os
import unittest

from vcfpy import Substitution

from mitoviz.classes import Locus, Variant, VcfParser

DATADIR = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), "data"
)
SAMPLE_HF_VCF = os.path.join(DATADIR, "sample_hf.vcf")


class TestLocus(unittest.TestCase):

    def setUp(self) -> None:
        self.dloop = Locus("DLOOP", 0)
        self.tf = Locus("TF", 1)
        self.rnr1 = Locus("RNR1", 2)
        self.nd1 = Locus("ND1", 6)

    def test_loc_type(self):
        self.assertEqual("reg", self.dloop.loc_type)
        self.assertEqual("trna", self.tf.loc_type)
        self.assertEqual("rrna", self.rnr1.loc_type)
        self.assertEqual("cds", self.nd1.loc_type)

    def test_width(self):
        self.assertEqual(0.4259388013760637, self.dloop.width)
        self.assertEqual(0.026953346611141286, self.tf.width)
        self.assertEqual(0.3621618685497012, self.rnr1.width)
        self.assertEqual(0.36292111775001507, self.nd1.width)

    def test_theta(self):
        self.assertEqual(0.0, self.dloop.theta)
        self.assertEqual(0.2264460739936025, self.tf.theta)
        self.assertEqual(0.42100368157402374, self.rnr1.theta)
        self.assertEqual(1.430045868791116, self.nd1.theta)

    def test_color(self):
        self.assertEqual("#ff7f50", self.dloop.color)
        self.assertEqual("#1e90ff", self.tf.color)
        self.assertEqual("#b22222", self.rnr1.color)
        self.assertEqual("#228b22", self.nd1.color)

    def test_text_ha(self):
        self.assertEqual("center", self.dloop.text_ha)
        self.assertEqual("center", self.tf.text_ha)
        self.assertEqual("right", self.rnr1.text_ha)
        self.assertEqual("right", self.nd1.text_ha)

    def test_text_va(self):
        self.assertEqual("bottom", self.dloop.text_va)
        self.assertEqual("bottom", self.tf.text_va)
        self.assertEqual("bottom", self.rnr1.text_va)
        self.assertEqual("bottom", self.nd1.text_va)

    def test_text_y(self):
        self.assertEqual(25.2, self.dloop.text_y)
        self.assertEqual(25.2, self.tf.text_y)
        self.assertEqual(25.2, self.rnr1.text_y)
        self.assertEqual(25.2, self.nd1.text_y)


class TestVariant(unittest.TestCase):

    def setUp(self) -> None:
        self.variant = Variant(
            reference="C",
            position=3308,
            alternate=Substitution("SNV", "A"),
            hf=0.3
        )

    def test__is_deletion(self):
        self.assertFalse(self.variant._is_deletion())

    def test__is_insertion(self):
        self.assertFalse(self.variant._is_insertion())

    def test_label(self):
        self.assertEqual("3308C>A", self.variant.label)

    def test_pos_x(self):
        self.assertEqual(1.2557981773190898, self.variant.pos_x)

    def test_pos_y(self):
        self.assertEqual(21.5, self.variant.pos_y)


class TestVcfParser(unittest.TestCase):

    def setUp(self) -> None:
        self.vcf = VcfParser(SAMPLE_HF_VCF)

    def test_variants(self):
        # Given/When
        variant = Variant(
            reference="C",
            position=8935,
            alternate=Substitution("SNV", "T"),
            hf=0.899
        )

        # Then
        self.assertIsInstance(self.vcf.variants, list)
        # self.assertIn(variant, self.vcf.variants)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Created by Roberto Preste
import os
import pytest
import unittest

from click.testing import CliRunner

from mitoviz import mitoviz
from mitoviz import cli

DATADIR = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), "data"
)
SAMPLE_HF_VCF = os.path.join(DATADIR, "sample_hf.vcf")
OUTPUT = os.path.join(DATADIR, "sample_hf_plot.png")


class TestCli(unittest.TestCase):

    def setUp(self) -> None:
        self.runner = CliRunner()

    def tearDown(self) -> None:
        try:
            os.remove("mitoviz.png")
            os.remove(OUTPUT)
        except FileNotFoundError:
            pass

    def test_cli_help(self):
        # Given/When
        result = self.runner.invoke(cli.main, ["--help"])

        # Then
        self.assertEqual(0, result.exit_code)
        self.assertIn("Show this message and exit.", result.output)

    def test_cli_plot(self):
        # Given/When
        result = self.runner.invoke(cli.main, [SAMPLE_HF_VCF])

        # Then
        self.assertEqual(0, result.exit_code)
        self.assertTrue(os.path.isfile("mitoviz.png"))

    def test_cli_plot_output(self):
        # Given/When
        result = self.runner.invoke(cli.main,
                                    [SAMPLE_HF_VCF,
                                     "--output", OUTPUT])

        # Then
        self.assertEqual(0, result.exit_code)
        self.assertTrue(os.path.isfile(OUTPUT))



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Created by Roberto Preste
import unittest

from mitoviz.utils import convert_hf, convert_nt


class TestUtils(unittest.TestCase):

    def test_convert_nt(self):
        # Given
        nt = 3308
        expected = 1.2557981773190898

        # When
        result = convert_nt(nt)

        # Then
        self.assertEqual(expected, result)

    def test_convert_hf(self):
        # Given
        hf = 0.3
        expected = 1.5

        # When
        result = convert_hf(hf)

        # Then
        self.assertEqual(expected, result)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Created by Roberto Preste
import sys
import click

from .plot import plot_vcf


@click.command()
@click.version_option()
@click.argument("input_vcf")
@click.option("--output", "-o", default="mitoviz.png",
              help="Output filename.")
def main(input_vcf, output):
    """ Plot variants on the human mitochondrial genome. """
    plot_vcf(in_vcf=input_vcf, save=True, output=output)

    return 0


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Created by Roberto Preste
import matplotlib.pyplot as plt

from .classes import Locus, VcfParser
from .constants import NAMES


def plot_mito():
    """ Return an axes object with the base mitochondrial genome plot. """
    fig = plt.figure(figsize=(20, 10), dpi=300)
    ax = fig.add_subplot(111, polar=True)
    # ax = plt.axes([0.025, 0.025, 0.95, 0.95], polar=True)
    loci = [Locus(name=name, index=index)
            for name, index in zip(NAMES, range(len(NAMES)))]
    thetas = [el.theta for el in loci]
    radii = [5.0] * 38
    widths = [el.width for el in loci]

    bars = plt.bar(thetas, radii, width=widths, bottom=20.0)
    for locus, bar in zip(loci, bars):
        bar.set_facecolor(locus.color)
        ax.annotate(locus.name,
                    xy=(locus.theta, locus.text_y),
                    ha=locus.text_ha, va=locus.text_va)

    ax.set_xticklabels([])
    ax.set_yticklabels([])
    ax.xaxis.grid(False)
    ax.yaxis.grid(False)
    ax.spines["polar"].set_visible(False)
    ax.set_theta_zero_location("N")

    return fig, ax


def plot_vcf(in_vcf: str,
             save: bool = False,
             output: str = "mitoviz.png") -> None:
    """ Plot variants from the given VCF file.

    Parameters
    ----------
    in_vcf : str
        Path of the input VCF file.
    save : bool
        If true, the final plot will be saved to a file.
    output : str
        Path of the output file where the plot will be saved
        (default: ./mitoviz.png).
    """
    fig, ax = plot_mito()

    vcf = VcfParser(in_vcf)
    for variant in vcf.variants:
        ax.scatter(variant.pos_x, variant.pos_y, c="black", s=20, zorder=20)

    if save:
        plt.savefig(output)

    return None



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Created by Roberto Preste


def convert_nt(nt: int) -> float:
    """ Convert a nucleotide position/width to a relative value.

    Used to calculate relative values required for polar plots.

    Parameters
    ----------
    nt : int
        Position/width in nucleotides.

    Returns
    -------
    float
        Relative value.
    """
    return (nt * 6.29) / 16569


def convert_hf(hf: float) -> float:
    """ Convert a variant's HF value from the [0.0, 1.0] range to [0.0, 5.0].

    Used to plot the variant on a polar plot.

    Parameters
    ----------
    hf : float
        Heteroplasmic fraction of the variant.

    Returns
    -------
    float
        Adjusted HF value.
    """
    return hf * 5



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Created by Roberto Preste
import pytest

import os


DATADIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), "data")


def main():
    """Create the test files needed."""
    pass


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Created by Roberto Preste
import vcfpy

from .constants import COLORS, NT_LENGTHS, TEXT_HA, TEXT_VA, TEXT_Y, TYPES
from .utils import convert_hf, convert_nt


class Locus:
    """ Class referring to a single mt locus.

    Used to create the base mitochondrial plot on which variants will
    be added on a separate layer.

    Attributes:
        name: name of the locus
        index: index of the locus in the mt genome (dloop = 0, tf = 1, etc.)
    """

    def __init__(self, name: str, index: int):
        self.name = name
        self.index = index

    @property
    def loc_type(self) -> str:
        """ The locus type (regulatory, coding, rRNA, tRNA). """
        return TYPES[self.index]

    @property
    def width(self) -> float:
        """ The relative width of the locus from its length in
            nucleotides.
        """
        return convert_nt(NT_LENGTHS[self.index])

    @property
    def theta(self) -> float:
        """ The position at which the locus will be plotted. """
        if self.index == 0:
            return 0.0
        elif self.index == 1:
            return convert_nt(NT_LENGTHS[0])/2 + self.width/2
        return (convert_nt(NT_LENGTHS[0])/2 + self.width/2
                + sum(map(convert_nt, NT_LENGTHS[1:self.index])))

    @property
    def color(self) -> str:
        """ The locus-type-specific color. """
        return COLORS[self.loc_type]

    @property
    def text_ha(self) -> str:
        """ The horizontal alignment for the text label. """
        return TEXT_HA[self.index]

    @property
    def text_va(self) -> str:
        """ The vertical alignment for the text label. """
        return TEXT_VA[self.index]

    @property
    def text_y(self) -> float:
        """ The y position for the text label. """
        return TEXT_Y[self.index]

    def __repr__(self):
        return "{}(name={}, index={})".format(
            self.__class__.__name__, self.name, self.index
        )


class Variant:
    """ Class used to store a given variant.

    Attributes:
        reference: reference allele of the variant
        position: position of the variant
        alternate: alternate allele of the variant
        hf: heteroplasmic fraction of the variant
    """

    def __init__(self,
                 reference: str,
                 position: int,
                 alternate: vcfpy.Substitution,
                 hf: float):
        self.reference = reference
        self.position = position
        self.alternate = alternate
        self.hf = hf

    def _is_deletion(self) -> bool:
        """ Check whether the current variant refers to a deletion. """
        # e.g. ref CTG | alt C
        return self.alternate.type == "DEL"

    def _is_insertion(self) -> bool:
        """ Check whether the current variant refers to an insertion. """
        # e.g. ref C | alt CTG
        return self.alternate.type == "INS"

    @property
    def label(self) -> str:
        """ Create the variant label for deletions, insertions and SNPs. """
        if self._is_deletion():
            label = "{}d".format(int(self.position) + 1)
        elif self._is_insertion():
            label = "{}.{}".format(
                self.position, self.alternate.value[len(self.reference):]
            )
        else:
            label = "{}{}>{}".format(self.position, self.reference,
                                     self.alternate.value)

        return label

    @property
    def pos_x(self) -> float:
        """ The x position of the variant on the mitochondrial genome plot. """
        return convert_nt(self.position)

    @property
    def pos_y(self) -> float:
        """ The y position of the variant on the mitochondrial genome plot,
            based on its heteroplasmic fraction.
        """
        return 20 + convert_hf(self.hf)

    def __repr__(self):
        return "{}(reference={}, position={}, alternate={}, hf={})".format(
            self.__class__.__name__, self.reference, self.position,
            self.alternate, self.hf
        )


class VcfParser:
    """ Class to read and parse the given VCF file.

    Attributes
        vcf_in: path of the input VCF file
    """

    def __init__(self, vcf_in: str):
        self.vcf_in = vcf_in
        self._variants = []
        self._reader = vcfpy.Reader.from_path(vcf_in)
        self.parse_variants()

    @property
    def variants(self):
        return self._variants

    @variants.setter
    def variants(self, value):
        if isinstance(value, list):
            self._variants.extend(value)
        elif isinstance(float, value):
            self._variants.append(value)
        else:
            raise ValueError("not allowed")

    def parse_variants(self):
        """ Read the variants from the input VCF file and parse them in the
            required format.
        """
        for record in self._reader:
            if "HF" in record.FORMAT:
                variants = [
                    Variant(record.REF, record.POS, alt,
                            record.calls[0].data["HF"][i])
                    for i, alt in enumerate(record.ALT)
                ]
            else:
                variants = [
                    Variant(record.REF, record.POS, alt, 0.5)
                    for alt in record.ALT
                ]
            self.variants = variants

    def __repr__(self):
        return "{}(vcf_in={})".format(
            self.__class__.__name__, self.vcf_in
        )
